import { SyntaxKind } from 'ts-morph';
export function hasUsagesInTs(declaration) {
    const referencingNodes = declaration.findReferencesAsNodes().filter(node => {
        const sourceFile = node.getSourceFile();
        return (!sourceFile.isDeclarationFile() &&
            !sourceFile.getBaseName().includes('.spec.ts'));
    });
    return referencingNodes.some(node => isReferecingNodeRelevant(node));
}
function isReferecingNodeRelevant(node) {
    const irrelevantNodeKinds = [
        SyntaxKind.ImportSpecifier,
        SyntaxKind.ExportSpecifier,
    ];
    const irrelevantParentNodeKinds = [
        SyntaxKind.ExportAssignment,
        SyntaxKind.ClassDeclaration,
        SyntaxKind.ImportSpecifier,
        SyntaxKind.ExportSpecifier,
    ];
    const isOfIrrelevantKind = irrelevantNodeKinds.includes(node.getKind());
    const hasParentOfIrrelevantKind = node.getParent() !== undefined &&
        irrelevantParentNodeKinds.includes(node.getParent().getKind());
    return ((!isOfIrrelevantKind &&
        !hasParentOfIrrelevantKind &&
        !isInNgModuleDecoratorCall(node)) ||
        node.getParent().isKind(SyntaxKind.PropertyAssignment));
}
function isInNgModuleDecoratorCall(node) {
    if (node.getParent() === undefined) {
        return false;
    }
    if (node.getParent().isKind(SyntaxKind.PropertyAssignment)) {
        return false;
    }
    if (node.getFirstAncestor(ancestor => ancestor.isKind(SyntaxKind.PropertyAssignment) &&
        ancestor.getName() === 'bootstrap') !== undefined) {
        return false;
    }
    return (node.getFirstAncestor(ancestor => ancestor.isKind(SyntaxKind.Decorator) &&
        ancestor.getFullName() === 'NgModule') !== undefined);
}
